#!/usr/bin/env python3

import sys

def extended_gcd(a, b):
    """
    Computes the greatest common divisor d of a and b
    and finds x and y such that d = a*x + b*y

    :param a: <int> integer
    :param b: <int> integer
    :return x y d: (<int>, <int>, <int>) such that a*x + b*y = d, where d = gcd(a, b)
    """

    if b == 0:
        return 1, 0, a
    else:
        x1, y1, d = extended_gcd(b, a % b)
        return y1, x1 - y1*int((a/b)), d

def main():
    if len(sys.argv) < 3:
        print(f"Usage: {sys.argv[0]} <a> <b>")
        exit()
    try:
        a, b = int(sys.argv[1]), int(sys.argv[2])
        x, y, d = extended_gcd(a, b)
        print(f"{a} * {x} + {b} * {y} = {d} = GCD({a}, {b})")
        
    except ValueError as e:
        print(f"[ERR] – Failed to convert arguments into ints")
        print(e)
        exit()    

if __name__ == "__main__":
    main()    
