#!/usr/bin/env python3

import sys

def square_and_multiply(x, y, n):
    """
     Computes x^y mod p using the following properties:
     1) x^y = x^(y/2) * x^(y/2)     if y is even
     2) x^y = x^(y/2) * x^(y/2) * x if y is odd
     3) x*y mod C = ((x mod C) * (y mod C)) mod C

     Complexity: O(m^3) where m = max {number of bits of x, number of bits of y, n}

    :param x: integer, base
    :param y: integer, exponent
    :param n: integer, modulo
    :return: x^y mod n
    """
    if y == 0:
        return 1

    r = square_and_multiply(x, int(y/2), n)

    if y % 2 == 0:
        return r*r % n
    else:
        return r*r*x % n

def main():
    if len(sys.argv) < 4:
        print(f"Usage: {sys.argv[0]} <x> <y> <n>")
        exit()
    try:
        x, y, n = int(sys.argv[1]), int(sys.argv[2]), int(sys.argv[3])
        result = square_and_multiply(x, y, n)
        print(f"Computed value {x}^{y} MOD {n} = {result}")
        
    except ValueError as e:
        print(f"[ERR] – Failed to convert arguments into ints")
        print(e)
        exit()    

if __name__ == "__main__":
    main()
